"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useEasing = void 0;
var react_1 = require("react");
var easings_1 = require("./easings");
var noop = function () { };
var identity = function (val) { return val; };
var formatter = function (f) { return ({
    format: f
}); };
function useEasing(_a) {
    var _b = _a.start, start = _b === void 0 ? 0 : _b, end = _a.end, duration = _a.duration, _c = _a.autoStart, autoStart = _c === void 0 ? true : _c, _d = _a.easingFn, easingFn = _d === void 0 ? easings_1.easeInQuad : _d, _e = _a.formatFn, formatFn = _e === void 0 ? identity : _e, _f = _a.onCleanUp, onCleanUp = _f === void 0 ? noop : _f, _g = _a.onPauseResume, onPauseResume = _g === void 0 ? noop : _g, _h = _a.onStart, onStart = _h === void 0 ? noop : _h, _j = _a.onEnd, onEnd = _j === void 0 ? noop : _j;
    var _k = react_1.useState(autoStart), trigger = _k[0], setTrigger = _k[1];
    var _l = react_1.useState(start), data = _l[0], setData = _l[1];
    var easingFnRef = react_1.useRef(easingFn);
    var dataRef = react_1.useRef(data);
    dataRef.current = data; // Update on every render
    var callbacks = react_1.useRef({ onCleanUp: onCleanUp, onPauseResume: onPauseResume, onStart: onStart, onEnd: onEnd });
    react_1.useEffect(function () {
        var raf;
        var callNewRaf = true;
        if (trigger) {
            // console.group("CountUp Effect starts");
            callbacks.current.onStart();
            // request animation frames receives time in milliseconds
            raf = window.requestAnimationFrame(function (startTime) {
                var endTime = startTime + duration * 1000;
                // what to use as starting point?
                var _start = dataRef.current;
                // get the total change
                var change = end - _start;
                // console.log("Animation Frames running with these settings:", {
                //   start: _start,
                //   currentRef: dataRef.current,
                //   end,
                //   change,
                //   startTime,
                //   endTime
                // });
                // console.groupEnd();
                setData(_start);
                var handler = function (time) {
                    if (time > endTime) {
                        setData(end);
                        return onEnd();
                    }
                    setData(function () {
                        return easingFnRef.current(time - startTime, _start, change, duration * 1000);
                    });
                    if (callNewRaf) {
                        raf = window.requestAnimationFrame(handler);
                    }
                    return noop;
                };
                raf = window.requestAnimationFrame(handler);
            });
        }
        else {
            // console.group("CountUp effect");
            // console.log("But it could not be started");
            // console.log("trigger", trigger);
            // console.groupEnd();
            callbacks.current.onPauseResume();
        }
        return function () {
            // console.group("CountUp Clean Up");
            // console.log("trigger: ", trigger);
            // console.log("CountUp: ", dataRef.current);
            // console.log("Animation Frame: ", raf);
            // console.groupEnd();
            callbacks.current.onCleanUp();
            callNewRaf = false;
            window.cancelAnimationFrame(raf);
        };
    }, [trigger, end, duration]);
    var userFormat = formatter(formatFn);
    return {
        value: userFormat.format(data),
        setTrigger: setTrigger
    };
}
exports.useEasing = useEasing;
exports.default = useEasing;
//# sourceMappingURL=index.js.map